"""
BlackboardSync
"""

# Copyright (C) 2023, Jacob Sánchez Pérez

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301, USA.

__all__ = [
    "__id__",
    "__title__",
    "__summary__",
    "__uri__",
    "__homepage__",
    "__author__",
    "__email__",
    "__publisher__",
    "__license__",
    "__license_spdx__",
    "__copyright__",
]

__id__ = "app.bbsync.BlackboardSync"
__title__ = "Blackboard Sync"

__summary__ = "Automatic Syncing Of Your Blackboard Content"
__uri__ = "https://github.com/sanjacob/BlackboardSync"
__homepage__ = "https://bbsync.app"

__author__ = "Jacob Sánchez"
__email__ = "jacobszpz@protonmail.com"
__publisher__ = "WeAreMagic"

__license__ = "GNU General Public License v2.0 only"
__license_spdx__ = "GPL-2.0-only"

__copyright__ = f"2024, {__author__}"
