"""
BlackboardSync Qt GUI.

~~~~~~~~~~~~~~~~~~~~~

A graphical interface using Qt for BlackboardSync.
Full documentation available at <https://blackboardsync.readthedocs.io>.

:copyright: (c) 2024, Jacob Sánchez Pérez.
:license: GPL v2.0, see LICENSE for more details.
"""

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301, USA.

from .SyncTrayIcon import SyncTrayIcon
from .SetupWizard import SetupWizard
from .LoginWebView import LoginWebView
from .SettingsWindow import SettingsWindow


__all__ = [
    'SetupWizard',
    'LoginWebView',
    'SettingsWindow',
    'SyncTrayIcon'
]
